@@ -3,7 +3,10 @@
 
     This file is part of qgpgme, the Qt API binding for gpgme
     Copyright (c) 2004,2008 Klarälvdalens Datakonsult AB
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
+    Copyright (c) 2022,2023 g10 Code GmbH
+    Software engineering by Ingo Klöcker <dev@ingo-kloecker.de>
 
     QGpgME is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -31,11 +34,22 @@
     your version.
 */
 
+#ifdef HAVE_CONFIG_H
+ #include "config.h"
+#endif
+
 #include "qgpgmelistallkeysjob.h"
 
+#include "listallkeysjob_p.h"
+
+#include "debug.h"
 #include "key.h"
 #include "context.h"
+#include "engineinfo.h"
+#include "global.h"
 #include "keylistresult.h"
+#include "qgpgme_debug.h"
+
 #include <gpg-error.h>
 
 #include <algorithm>
@@ -47,21 +61,48 @@
 using namespace QGpgME;
 using namespace GpgME;
 
+namespace
+{
+
+class QGpgMEListAllKeysJobPrivate : public ListAllKeysJobPrivate
+{
+    QGpgMEListAllKeysJob *q = nullptr;
+
+public:
+    QGpgMEListAllKeysJobPrivate(QGpgMEListAllKeysJob *qq)
+        : q{qq}
+    {
+    }
+
+    ~QGpgMEListAllKeysJobPrivate() override = default;
+
+private:
+    void start() override
+    {
+        q->run();
+    }
+};
+
+}
+
 QGpgMEListAllKeysJob::QGpgMEListAllKeysJob(Context *context)
     : mixin_type(context),
       mResult()
 {
+    setJobPrivate(this, std::unique_ptr<QGpgMEListAllKeysJobPrivate>{new QGpgMEListAllKeysJobPrivate{this}});
     lateInitialization();
 }
 
 QGpgMEListAllKeysJob::~QGpgMEListAllKeysJob() {}
 
-static KeyListResult do_list_keys(Context *ctx, std::vector<Key> &keys, bool secretOnly)
+namespace {
+
+static KeyListResult do_list_keys_legacy(Context *ctx, std::vector<Key> &keys, bool secretOnly)
 {
 
-    const char **pat = 0;
+    const char **pat = nullptr;
     if (const Error err = ctx->startKeyListing(pat, secretOnly)) {
-        return KeyListResult(0, err);
+        return KeyListResult(nullptr, err);
     }
 
     Error err;
@@ -76,9 +117,6 @@ static KeyListResult do_list_keys(Context *ctx, std::vector<Key> &keys, bool sec
     return result;
 }
 
-namespace
-{
-
 template <typename ForwardIterator, typename BinaryPredicate>
 ForwardIterator unique_by_merge(ForwardIterator first, ForwardIterator last, BinaryPredicate pred)
 {
@@ -98,8 +136,6 @@ ForwardIterator unique_by_merge(ForwardIterator first, ForwardIterator last, Bin
     return ++dest;
 }
 
-}
-
 static void merge_keys(std::vector<Key> &merged, std::vector<Key> &pub, std::vector<Key> &sec)
 {
     merged.reserve(pub.size() + sec.size());
@@ -113,15 +149,15 @@ static void merge_keys(std::vector<Key> &merged, std::vector<Key> &pub, std::vec
                  merged.end());
 }
 
-static QGpgMEListAllKeysJob::result_type list_keys(Context *ctx, bool mergeKeys)
+static QGpgMEListAllKeysJob::result_type list_keys_legacy(Context *ctx, bool mergeKeys)
 {
     std::vector<Key> pub, sec, merged;
     KeyListResult r;
 
-    r.mergeWith(do_list_keys(ctx, pub, false));
+    r.mergeWith(do_list_keys_legacy(ctx, pub, false));
     std::sort(pub.begin(), pub.end(), ByFingerprint<std::less>());
 
-    r.mergeWith(do_list_keys(ctx, sec, true));
+    r.mergeWith(do_list_keys_legacy(ctx, sec, true));
     std::sort(sec.begin(), sec.end(), ByFingerprint<std::less>());
 
     if (mergeKeys) {
@@ -132,15 +168,66 @@ static QGpgMEListAllKeysJob::result_type list_keys(Context *ctx, bool mergeKeys)
     return std::make_tuple(r, merged, sec, QString(), Error());
 }
 
+static KeyListResult do_list_keys(Context *ctx, std::vector<Key> &keys)
+{
+    const unsigned int keyListMode = ctx->keyListMode();
+    ctx->addKeyListMode(KeyListMode::WithSecret);
+
+    const char **pat = nullptr;
+    if (const Error err = ctx->startKeyListing(pat)) {
+        ctx->setKeyListMode(keyListMode);
+        return KeyListResult(nullptr, err);
+    }
+
+    Error err;
+    do {
+        keys.push_back(ctx->nextKey(err));
+    } while (!err);
+
+    keys.pop_back();
+
+    const KeyListResult result = ctx->endKeyListing();
+    ctx->setKeyListMode(keyListMode);
+
+    ctx->cancelPendingOperation();
+    return result;
+}
+
+static QGpgMEListAllKeysJob::result_type list_keys(Context *ctx, bool mergeKeys, ListAllKeysJob::Options options)
+{
+    if (GpgME::engineInfo(GpgME::GpgEngine).engineVersion() < "2.1.0") {
+        return list_keys_legacy(ctx, mergeKeys);
+    }
+
+    if (options & ListAllKeysJob::DisableAutomaticTrustDatabaseCheck) {
+        auto err = ctx->setFlag("no-auto-check-trustdb", "1");
+        if (err) {
+            // ignore error, but log a warning
+            qCWarning(QGPGME_LOG) << "Setting context flag no-auto-check-trustdb failed:" << err;
+        }
+    }
+
+    std::vector<Key> keys;
+    KeyListResult r = do_list_keys(ctx, keys);
+    std::sort(keys.begin(), keys.end(), ByFingerprint<std::less>());
+
+    std::vector<Key> sec;
+    std::copy_if(keys.begin(), keys.end(), std::back_inserter(sec), [](const Key &key) { return key.hasSecret(); });
+
+    return std::make_tuple(r, keys, sec, QString(), Error());
+}
+
+}
+
 Error QGpgMEListAllKeysJob::start(bool mergeKeys)
 {
-    run(std::bind(&list_keys, std::placeholders::_1, mergeKeys));
+    run(std::bind(&list_keys, std::placeholders::_1, mergeKeys, options()));
     return Error();
 }
 
 KeyListResult QGpgMEListAllKeysJob::exec(std::vector<Key> &pub, std::vector<Key> &sec, bool mergeKeys)
 {
-    const result_type r = list_keys(context(), mergeKeys);
+    const result_type r = list_keys(context(), mergeKeys, options());
     resultHook(r);
     pub = std::get<1>(r);
     sec = std::get<2>(r);
